@@ -53,7 +53,6 @@ print_pnm (FILE * fptr)
 	     gMessages[LOG_START]);
     wrap_syslog (LOG_INFO,"%s",syslog_message); 
 
-
     ppa_init_job (&printer);
 
     while (initImage (&image, fptr))
@@ -156,7 +155,8 @@ readPath(char *input )
 	    a user-specifed file *File_path may later be opened;
 	    should any extra validation of this be performed now ?
 	  */
-	  path = File_path;
+	  /* remember to free(path) after use! */
+	  path = strdup(File_path);
 	  return path;
 	}
     }
@@ -177,7 +177,7 @@ readPath(char *input )
 void
 set_printer_specific_defaults (void)
 {
-    if (printer.version != HP7X0 &&
+    if (printer.version != NOPRINTER && printer.version != HP7X0 &&
 	printer.version != HP820 && printer.version != HP1000)
     {
       snprintf(syslog_message,message_size, "set_printer_defaults(): %s",
@@ -222,6 +222,7 @@ show_usage (char *prog)
     printf ("    --bw            forces black/white output\n");
     printf ("    -d              dumps the configuration to stdout\n");
     printf ("    --eco           Econofast mode (uses less ink)\n");
+    printf ("    --dpi300        Treat input resolution as 300dpi instead of 600dpi\n");
     printf ("    -f <cfgfile>    read printer settings from configuration file <cfgfile>\n");
     printf ("    -F <gammaFile>  read color correction table from file <gammaFile>\n");
     printf ("    --fd            use fast dithering mode\n");
@@ -265,15 +266,21 @@ show_usage (char *prog)
     printf
       ("  the corresponding command-line parameters.\n");
     printf
-      ("  Other keys that can be set in the configuration file include:\n");
+      ("  Other keys that can be only set in the configuration file include:\n");
     printf
-      ("  'silent', 'verbose', 'unimode', 'blackshear', 'colorshear',\n"); 
+      ("     'silent', 'verbose', 'log_info',\n");
+    printf 
+      ("     'unimode', 'blackshear', 'colorshear',\n"); 
     printf
-      ("  'RedGammaIdx','BlueGammaIdx', 'GreenGammaIdx',\n");
+      ("     'RedGammaIdx','BlueGammaIdx', 'GreenGammaIdx',\n");
     printf
-      ("  'GammaR', 'GammaG', 'GammaB', 'ColOffsX', 'ColOffsY',\n");
+      ("     'GammaR', 'GammaG', 'GammaB',\n");
     printf
-      ("  'black_ink', 'color_ink', 'cyan_ink', 'magenta_ink', 'yellow_ink';\n");
+      ("     'ColOffsX', 'ColOffsY',\n");
+    printf
+      ("     'black_ink', 'color_ink', 'cyan_ink', 'magenta_ink', 'yellow_ink',\n");
+    printf
+      ("     'compression';\n");
     printf
       ("  see documentation in the sample configuration file for their usage.\n\n"); 
     printf
@@ -293,18 +300,9 @@ show_usage (char *prog)
 void
 parm_version (char *arg)
 {
-
-    if (!strncasecmp (arg, "error", 5))
-    {
-      wrap_closelog ();
-      wrap_openlog ("pnm2ppa", 1);
-      snprintf(syslog_message,message_size,"parm_version(): %s", 
-	       gMessages[E_CONFIG]);
-      wrap_syslog (LOG_CRIT,"%s",syslog_message); 
-
-      exit (1);
-    }
     
+
+
     if (!strcasecmp (arg, "hp720") || !strcmp (arg, "720") ||
 	!strcasecmp (arg, "hp722") || !strcmp (arg, "722") ||
 	!strcasecmp (arg, "hp710") || !strcmp (arg, "710") ||
@@ -329,7 +327,10 @@ void
 parm_iversion (int arg)
 {
     switch (arg)
-    {
+    {      
+    case 0:
+	printer.version = NOPRINTER;
+	break;
     case 720:
     case 722:
     case 710:
@@ -530,6 +531,15 @@ read_config_file (char *fname)
 		  wrap_closelog();
 		}
 	    }
+	  else if (!strncmp (key, "log_info", len))
+	    {
+	      if (system_config_file) {
+		if ( value )
+		  gLogInfo = true ;
+		else
+		  gLogInfo = false ;
+	      }
+	    }
 	  else if (!strncmp (key, "verbose", len))
 	    {
 	      gVerbose  = true;
@@ -569,6 +579,13 @@ read_config_file (char *fname)
 		  exit (1);
 		}
 	    }
+	  else if (!strncmp (key, "compression", len))
+	    {
+	      if ( value )
+		gCompress = true ;
+	      else
+		gCompress = false ;
+	    }
 	  else
 	    {
 	      snprintf(syslog_message,message_size,"read_config_file(): %s (%s %d)\n",
@@ -612,9 +629,9 @@ main (int argc, char *argv[])
   char c;
   BOOLEAN get_Gamma = true;
 
-  char *gammaFile=NULL; 
-  char *pnm_inputFile=NULL;
-  char *ppa_outputFile=NULL;
+  char gammaFile[MAXPATHLEN];
+  char pnm_inputFile[MAXPATHLEN];
+  char ppa_outputFile[MAXPATHLEN];
   BOOLEAN use_stdin = false;
   BOOLEAN use_stdout = false;
   BOOLEAN got_Input = false;
@@ -625,11 +642,11 @@ main (int argc, char *argv[])
   
   system_config_file = true;
 
-#ifdef __BEOS__
+#ifdef __NO_SYSLOG__
   gSilent = true;
 #endif
 
-  gammaFile = defaultgammafile;
+  strncpy(gammaFile,defaultgammafile,MAXPATHLEN);
   
   /* activate syslog */
   if (!(gSilent))
@@ -663,6 +680,9 @@ main (int argc, char *argv[])
   gColorMode = true;
   gFastMode = false;
   gUnimode = false;
+  g300 = false;
+  gCompress = true;
+  gLogInfo = false;       /* no  LOG_INFO syslog messages by default */ 
   gBlack = true;
   gCyan = true;
   gMagenta = true;
@@ -680,7 +700,7 @@ main (int argc, char *argv[])
 
   system_config_file = false;
 
-  // read through the optional arguments before tring to find filenames
+  /* read through the optional arguments before tring to find filenames */
   
   while ((c = getopt_long (argc, argv, "F:B:b:df:ghi:l:o:pr:s:t:v:x:y:",
 			   long_opts, &opt_index)) != EOF)
@@ -688,39 +708,40 @@ main (int argc, char *argv[])
       switch (c)
 	{
 	case 0:
-	  // long options
+	  /* long options */
 	  switch (opt_index)
 	    {
 	    case 0:
-	      // --bw               Black & White 
+	      /* --bw               Black & White */ 
 	      gColorMode = false;
-	      // gEcoMode = false;   //this is not needed here (?) duncan
+#if 0
+	      gEcoMode = false;   /*this is not needed here (?) duncan*/
+#endif
 	      gMaxPass = 1;
 	      break;
 	      
 	    case 1:
-	      // --fd               Fast Dither
+	      /* --fd               Fast Dither*/
 	      gFastMode = true;
 	      gMaxPass = 1;
 	      break;
 	      
 	    case 2:
-	      // --version
-	      snprintf(syslog_message,message_size, "%s  %s  %s  %s %s\n",
+	      /* --version (output to stderr, not syslog) */
+	      fprintf(stderr,"%s  %s  %s  %s %s\n",
 		      *argv, gMessages[LOG_VERSION], 
 		       VERSION, __DATE__, __TIME__);
-	      wrap_syslog (LOG_INFO,"%s",syslog_message); 
 	      exit (0);
 	      break;
 	      
 	    case 3:
-	      // --help
+	      /* --help */
 	      show_usage (*argv);
 	      return 0;
 	      break;
 
 	    case 4:
-	      // --verbose
+	      /* --verbose */
 	      gVerbose = true;
 	      if (!(gSilent))
 		{
@@ -730,24 +751,29 @@ main (int argc, char *argv[])
 	      break;
 	      
 	    case 5:
-	      // --eco 
+	      /* --eco */
 	      gEcoMode = true;
 	      break;
 	      
 	    case 6:
-	      // --noGamma
+	      /* --noGamma */
 	      get_Gamma = false;
 	      break;
 	      
 	    case 7:
-	      // --uni
+	      /* --uni */
 	      gUnimode = true;
 	      break;
 	      
 	    case 8:
-	      // --bi
+	      /* --bi */
 	      gUnimode=false;
 	      break;
+
+	    case 9:
+	      /* --dpi300 */
+	      g300=true;
+	      break;
 	      
 	    default:
 	      snprintf(syslog_message,message_size,"main (): %s",
@@ -774,19 +800,23 @@ main (int argc, char *argv[])
 	  
 	case 'f':
 	  {
-	    char *configFile;
+	    char *configFile=NULL;
 	    configFile = readPath (optarg);
-	    if (configFile)
+	    if (configFile) {
 	      read_config_file (configFile);
+	      free(configFile);
+	    }  
 	  }
 	  break;
 	  
 	case 'F':
 	  {
-	    char *path;
+	    char *path=NULL;
 	    path = readPath (optarg);
-	    if (path)
-	      gammaFile = path;
+	    if (path) {
+	      strncpy(gammaFile,path,MAXPATHLEN);
+	      free(path);
+	    }
 	  }
 	  break;
 	  
@@ -807,11 +837,12 @@ main (int argc, char *argv[])
 	    }
 	  else
 	    {
-	      char *path;
+	      char *path=NULL;
 	      path = readPath (optarg);
 	      if (path)
 		{
-		  pnm_inputFile = path;
+		  strncpy(pnm_inputFile,path,MAXPATHLEN);
+		  free(path);
 		  use_stdin = false;
 		  got_Input = true;
 		}
@@ -830,11 +861,12 @@ main (int argc, char *argv[])
 	    }
 	  else
 	    {
-	      char *path;
+	      char *path=NULL;
 	      path = readPath(optarg);
 	      if (path)
 		{
-		  ppa_outputFile = path;		  
+		  strncpy(ppa_outputFile,path,MAXPATHLEN);
+		  free(path);
 		  use_stdout = false ;
 		  got_Output = true;
 		}
@@ -986,11 +1018,11 @@ main (int argc, char *argv[])
 	}
     }
   
-  
-  // if the user doesn't specify an input or output file, we make like a pipe
+  /* if no input is specified, we terminate */
+  /* if the user doesn't specify  output file, we make like a pipe*/
   
   if (in == NULL)
-	in = stdin;
+    exit(0);    
   if (out == NULL)
     out = stdout;
   
@@ -1097,9 +1129,17 @@ main (int argc, char *argv[])
 	
     }
 
+    /* if printer version is set to NOPRINTER in pnm2ppa.conf  */
+    if (printer.version == NOPRINTER) {
+      snprintf(syslog_message,message_size,"pnm2ppa.conf: version 0; %s", 
+	       gMessages[E_PPA_UNKNOWN]);
+      wrap_syslog (LOG_CRIT,"%s",syslog_message); 
+
+      exit (1);
+    }
+
     /* now do the work! */
 
     return print_pnm (in);
 }
 
-
