@@ -344,6 +344,10 @@ compress (unsigned char *in, int num_lines_d2, int final_len,
 {
     unsigned char *out = iout;
     int I, len = num_lines_d2;
+    unsigned compressed_len = 0;
+
+
+    /* abandon compression if datasize expands!  */
 
     for (I = 0; I < final_len; I += num_lines_d2, in += num_lines_d2)
     {
@@ -367,12 +371,19 @@ compress (unsigned char *in, int num_lines_d2, int final_len,
 		{
 		    /* Max is 128 */
 		    *out++ = 0x00;
+		    compressed_len++;
+		    if (compressed_len > final_len)
+		      return 0;
 		    i += 128;
 		    dup_len -= 128;
 		}
 		if (dup_len >= 1)
 		{
 		    *out++ = dup_len;
+		    compressed_len++;
+		    if (compressed_len > final_len)
+		      return 0;
+
 		    i += dup_len;
 		}
 		/* See if we have enough non-zeros to be worth compressing. */
@@ -386,6 +397,10 @@ compress (unsigned char *in, int num_lines_d2, int final_len,
 		    /* Max is 64 */
 		    *out++ = (char) 0x80;
 		    *out++ = (char) in[i];
+		    compressed_len +=2;
+		    if (compressed_len > final_len)
+		      return 0;
+
 		    i += 64;
 		    dup_len -= 64;
 		}
@@ -393,6 +408,10 @@ compress (unsigned char *in, int num_lines_d2, int final_len,
 		{
 		    *out++ = dup_len + 0x80;
 		    *out++ = in[i];
+		    compressed_len +=2;
+		    if (compressed_len > final_len)
+		      return 0;
+
 		    i += dup_len;
 		}
 	    }
@@ -430,10 +449,14 @@ compress (unsigned char *in, int num_lines_d2, int final_len,
 		    int j;
 
 		    *out++ = (char) 0xc0;
+		    compressed_len ++;
 		    for (j = i; j < i + 64; j++)
 		    {
 			*out++ = in[j];
 		    }
+		    compressed_len += 64;
+		    if (compressed_len > final_len)
+		      return 0;
 		    i += 64;
 		    lit_len -= 64;
 		}
@@ -442,15 +465,22 @@ compress (unsigned char *in, int num_lines_d2, int final_len,
 		    int j;
 
 		    *out++ = lit_len + 0xc0;
+		    compressed_len ++;
 		    for (j = i; j < i + lit_len; j++)
 		    {
 			*out++ = in[j];
 		    }
+		    compressed_len += lit_len;
+		    if (compressed_len > final_len)
+		      return 0;
 		    i += lit_len;
 		}
 	    }
 	}
     }
+    if (compressed_len > final_len)
+      return 0;
+
     return out - iout;
 }
 
@@ -475,215 +505,205 @@ packLong (unsigned long x, unsigned char *y)
 #endif
 }
 
-//#define do_compress_data (1)
+/*#define do_compress_data (1) */
 
 void
 ppa_print_sweep (ppaPrinter_t * printer, ppaSweepData_t * data)
 {
-    unsigned char *pc, *tpc;
-    unsigned i, datasize = data->data_size;
-    unsigned char sweep_packet[144];
-    int sweep_packet_size;
-    unsigned short HP7X0constants[] = { 0x8ca0, 0x4650, 0x12c0 };
-    unsigned short HP820constants[] = { 0x4650, 0x1c20, 0x0960 };
-    unsigned short HP1000constants[] = { 0x4650, 0x2328, 0x0708 };
-    unsigned short *constants;
-    int nozzle_data_size;
-    int MF;			/* Multiplicative Factor -- quick hack */
-    int k;
-
-    unsigned char do_compress_data;
-
-    DPRINTF("ppa_print_sweep: Dir: %d Color: %d Size: %d Vpos: %d Hpos: %d,%d\n", 
-	    data->direction, data->in_color, data->data_size, data->vertical_pos, 
-	    data->left_margin, data->right_margin );
-    assert( ((data->direction == left_to_right) || (data->direction == right_to_left)) );
-
-    if (gColorMode)
-	do_compress_data = 0;
-    else
-	do_compress_data = 1;
-
-    // DPRINTF("*********k up to %d\n",(data->in_color == 1 ? gMaxPass : 1));
-    for (k = 0; k < (data->in_color == 1 ? gMaxPass : 1); k++)
+  unsigned char *pc, *tpc;
+  unsigned i, datasize = data->data_size;
+  unsigned char sweep_packet[144];
+  int sweep_packet_size;
+  unsigned short HP7X0constants[] = { 0x8ca0, 0x4650, 0x12c0 };
+  unsigned short HP820constants[] = { 0x4650, 0x1c20, 0x0960 };
+  unsigned short HP1000constants[] = { 0x4650, 0x2328, 0x0708 };
+  unsigned short *constants;
+  int nozzle_data_size;
+  int MF;			/* Multiplicative Factor -- quick hack */
+  
+  unsigned char do_compress_data;
+  
+  DPRINTF("ppa_print_sweep(): Dir: %d Color: %d Size: %d Vpos: %d Hpos: %d,%d\n", 
+	  data->direction, data->in_color, data->data_size, data->vertical_pos, 
+	  data->left_margin, data->right_margin );
+  assert( ((data->direction == left_to_right) || (data->direction == right_to_left)) );
+  
+  if (data->in_color && !gCompress )
+    do_compress_data = 0;
+  else
+    do_compress_data = 1;
+  
+  
+  
+  pc = data->image_data;
+  
+  DPRINTF(" ppa_print_sweep(): datasize= %d\n", datasize);
+  
+  if (do_compress_data) {
+    unsigned compressed_datasize;
+    if (!(pc = malloc ((datasize / 64 + 1) * 65)))
+      {
+	snprintf(syslog_message,message_size, "ppa_print_sweep(): %s",   
+		 gMessages[E_PPA_BADMALLOC]);
+	wrap_syslog (LOG_CRIT,"%s",syslog_message);
+	  exit (-1);
+      }
+    compressed_datasize =
+      compress (data->image_data,
+		data->nozzle_data->pins_used_d2, datasize, pc);
+    if (!compressed_datasize) {
+      /* compression would increase datsize ! */
+      free (pc);
+	  do_compress_data = 0;
+	  pc = data->image_data;
+    } else {
+      datasize = compressed_datasize;
+      DPRINTF(" ppa_print_sweep():compressed datasize= %d\n", datasize);
+    }
+    
+  }
+  
+  /* send image data 16k at a time */
+  for (i = 0, tpc = pc; i < datasize; tpc += 16384, i += 16384)
     {
-	// DPRINTF("********step %d\n",k);
-	pc = data->image_data[k];
-
-	if (do_compress_data)
-	{
-	    if (!(pc = malloc ((datasize / 64 + 1) * 65)))
-	    {
-	      snprintf(syslog_message,message_size, "ppa_print_sweep(): %s",   
-		       gMessages[E_PPA_BADMALLOC]);
-	      wrap_syslog (LOG_CRIT,"%s",syslog_message);
-		exit (-1);
-	    }
-	    datasize =
-		compress (data->image_data[k],
-			  data->nozzle_data->pins_used_d2, datasize, pc);
-	}
-
-	/* send image data 16k at a time */
-	for (i = 0, tpc = pc; i < datasize; tpc += 16384, i += 16384)
-	{
-	    vlink_put (printer->fptr, 0, ((datasize - i) > 16384) ? 16384 : (datasize - i), tpc);
-	}
-
-	/* memory leak fix courtesy of John McKown */
-	if (do_compress_data)
-	    free (pc);
-
-	/* construct sweep packet */
-	switch (printer->version)
-	{
-	case HP820:
-	    constants = HP820constants;
-	    MF = 1;
-	    break;
-	case HP7X0:
-	    constants = HP7X0constants;
+      vlink_put (printer->fptr, 0, ((datasize - i) > 16384) ? 16384 : (datasize - i), tpc);
+    }
+  
+    
+  /* memory leak fix courtesy of John McKown */
+  if (do_compress_data && pc){
+    free (pc);
+    pc = NULL;
+  }
+  
+  /* construct sweep packet */
+  switch (printer->version) {
+  case HP820:
+    constants = HP820constants;
+    MF = 1;
+    break;
+  case HP7X0:
+    constants = HP7X0constants;
 	    MF = 2;
 	    break;
-	case HP1000:
-	    constants = HP1000constants;
-	    MF = 1;
-	    break;
-	default:
-	      snprintf(syslog_message,message_size, "ppa_print_sweep(): %s",
-		    gMessages[E_PPA_UNKNOWN]);
+  case HP1000:
+    constants = HP1000constants;
+    MF = 1;
+    break;
+  default:
+    snprintf(syslog_message,message_size, "ppa_print_sweep(): %s",
+	     gMessages[E_PPA_UNKNOWN]);
 	      wrap_syslog (LOG_CRIT,"%s",syslog_message);
-	    return;
-	}
-
-	sweep_packet[0] = 0;
-	sweep_packet[1] = do_compress_data;
-	sweep_packet[2] = data->direction == right_to_left ? 1 : 2;
-	sweep_packet[3] = data->in_color ? 14 : 1;
-
-	packLong (datasize, sweep_packet + 4);
-
-	memset (sweep_packet + 8, 0, 8);
-
-	if (data->in_color)
-	    packLong (MF * (data->vertical_pos +
-				/*96+ */ printer->y_offset) /*+ 1*/,
-			  sweep_packet + 16);
-	else
-	    packLong (MF *
-			  (data->vertical_pos +
-			   printer->y_offset), sweep_packet + 16);
-	packShort (constants[0], sweep_packet + 20);
-	packShort (MF *
-		   (data->left_margin + printer->x_offset) /*+ 1*/,
-		      sweep_packet + 22);
-	packShort (MF *
-		   (data->right_margin + printer->x_offset) /*+ 1*/,
-		      sweep_packet + 24);
-	if (data->in_color)
-	    packShort (0x2ee0, sweep_packet + 26);
-	else
-	    packShort (constants[1], sweep_packet + 26);	/* 2ee0 in color on 820 */
-	packShort (constants[2], sweep_packet + 28);
-	packShort (0x100, sweep_packet + 30);
-
-
-	if ((data->in_color) && (k < gMaxPass - 1))
-	{
-	    sweep_packet[32] = data->direction == right_to_left ? 1 : 2;
-	    sweep_packet[33] = data->in_color ? 14 : 1;
-	    packLong (MF *
-			  (data->vertical_pos +
-			   printer->y_offset), sweep_packet + 34);
-	    packShort (MF *
-			  (data->left_margin +
-			   printer->x_offset), sweep_packet + 38);
-	    packShort (MF *
-			  (data->right_margin +
-			   printer->x_offset), sweep_packet + 40);
-	    packShort (0x2ee0, sweep_packet + 42);
-	    packShort (constants[2], sweep_packet + 44);
-	}
-	else
-	{
-	    if (data->next)
-	    {
-		sweep_packet[32] =
-		    data->next->direction == right_to_left ? 1 : 2;
-		sweep_packet[33] = data->next->in_color ? 14 : 1;
-		packLong (MF *
-			      (data->next->vertical_pos +
-			       printer->y_offset), sweep_packet + 34);
-		packShort (MF *
-			      (data->next->left_margin +
-			       printer->x_offset), sweep_packet + 38);
-		packShort (MF *
-			      (data->next->right_margin +
-			       printer->x_offset), sweep_packet + 40);
-		if (data->in_color)
-		    packShort (0x2ee0, sweep_packet + 42);
-		else
-		    packShort (constants[1], sweep_packet + 42);	/* 2ee0 in color on 820 */
+	      return;
+  }
+  
+  sweep_packet[0] = 0;
+  sweep_packet[1] = do_compress_data;
+  sweep_packet[2] = data->direction == right_to_left ? 1 : 2;
+  sweep_packet[3] = data->in_color ? 14 : 1;
+  
+  packLong (datasize, sweep_packet + 4);
+  
+  memset (sweep_packet + 8, 0, 8);
+  
+  if (data->in_color)
+    packLong (MF * (data->vertical_pos +
+		    /*96+ */ printer->y_offset) /*+ 1*/,
+	      sweep_packet + 16);
+  else
+    packLong (MF *
+	      (data->vertical_pos +
+	       printer->y_offset), sweep_packet + 16);
+  packShort (constants[0], sweep_packet + 20);
+  packShort (MF *
+	     (data->left_margin + printer->x_offset) /*+ 1*/,
+	     sweep_packet + 22);
+  packShort (MF *
+	     (data->right_margin + printer->x_offset) /*+ 1*/,
+	     sweep_packet + 24);
+  if (data->in_color)
+    packShort (0x2ee0, sweep_packet + 26);
+  else
+    packShort (constants[1], sweep_packet + 26);	/* 2ee0 in color on 820 */
+  packShort (constants[2], sweep_packet + 28);
+  packShort (0x100, sweep_packet + 30);
+  
+  
+  if (data->next) {
+    sweep_packet[32] =
+      data->next->direction == right_to_left ? 1 : 2;
+    sweep_packet[33] = data->next->in_color ? 14 : 1;
+    packLong (MF *
+	      (data->next->vertical_pos +
+	       printer->y_offset), sweep_packet + 34);
+    packShort (MF *
+	       (data->next->left_margin +
+		printer->x_offset), sweep_packet + 38);
+    packShort (MF *
+	       (data->next->right_margin +
+		printer->x_offset), sweep_packet + 40);
+    if (data->in_color)
+      packShort (0x2ee0, sweep_packet + 42);
+    else
+      packShort (constants[1], sweep_packet + 42);	/* 2ee0 in color on 820 */
 		packShort (constants[2], sweep_packet + 44);
-	    }
-	    else
-		memset (sweep_packet + 32, 0, 14);
-	    sweep_packet[46] = 8;
-	}
-	nozzle_data_size = data->nozzle_data_size;
-	if (nozzle_data_size > 6)
-	{
-	  snprintf(syslog_message,message_size, "ppa_print_sweep(): %s %d\n",
-		   gMessages[W_PPA_NOZZLE], nozzle_data_size); 
-	  wrap_syslog (LOG_WARNING,"%s",syslog_message);
-
-	    nozzle_data_size = 6;
-	}
-	sweep_packet[47] = nozzle_data_size;
-
-	for (pc = sweep_packet + 48, i = 0;
-	     i < nozzle_data_size; i++, pc += 16)
-	{
-	    packShort (data->nozzle_data[i].DPI, pc);
-	    packShort (data->nozzle_data[i].pins_used_d2, pc + 2);
-	    packShort (data->nozzle_data[i].unused_pins_p1, pc + 4);
-	    packShort (data->nozzle_data[i].first_pin, pc + 6);
-	    packShort (data->nozzle_data[i].pins_used_d2, pc + 8);
-	    packShort (MF * (data->nozzle_data[i].left_margin + printer->x_offset),
-			  pc + 10);
-	    packShort (MF * (data->nozzle_data[i].right_margin + printer->x_offset),
-			  pc + 12);
-	    pc[14] = data->nozzle_data[i].nozzle_delay;
-	    pc[15] = 0;
-	}
-
-	sweep_packet_size = data->in_color ? 144 : 80;
-
-	/* send sweep packet */
-	switch (printer->version)
-	{
-	case HP820:
-	    scp_put (printer->fptr, 18, 1, 7, sweep_packet_size, sweep_packet);
-	    break;
-	case HP7X0:
-	    scp2_put (printer->fptr, 0x0180, sweep_packet_size,
+  }
+  else
+    memset (sweep_packet + 32, 0, 14);
+  sweep_packet[46] = 8;
+  
+  nozzle_data_size = data->nozzle_data_size;
+  if (nozzle_data_size > 6) {
+    snprintf(syslog_message,message_size, "ppa_print_sweep(): %s %d\n",
+	     gMessages[W_PPA_NOZZLE], nozzle_data_size); 
+    wrap_syslog (LOG_WARNING,"%s",syslog_message);
+    
+    nozzle_data_size = 6;
+  }
+  sweep_packet[47] = nozzle_data_size;
+  
+  for (pc = sweep_packet + 48, i = 0;
+       i < nozzle_data_size; i++, pc += 16) {
+    packShort (data->nozzle_data[i].DPI, pc);
+    packShort (data->nozzle_data[i].pins_used_d2, pc + 2);
+    packShort (data->nozzle_data[i].unused_pins_p1, pc + 4);
+    packShort (data->nozzle_data[i].first_pin, pc + 6);
+    packShort (data->nozzle_data[i].pins_used_d2, pc + 8);
+    packShort (MF * (data->nozzle_data[i].left_margin + printer->x_offset),
+	       pc + 10);
+    packShort (MF * (data->nozzle_data[i].right_margin + printer->x_offset),
+	       pc + 12);
+    pc[14] = data->nozzle_data[i].nozzle_delay;
+    pc[15] = 0;
+  }
+  
+  sweep_packet_size = data->in_color ? 144 : 80;
+  
+  
+  /* send sweep packet */
+  switch (printer->version) {
+  case HP820:
+    scp_put (printer->fptr, 18, 1, 7, sweep_packet_size, sweep_packet);
+    break;
+  case HP7X0:
+    scp2_put (printer->fptr, 0x0180, sweep_packet_size,
 		      7, 1, datasize, sweep_packet);
-	    break;
-	case HP1000:
-	    scp3_put (printer->fptr, 0x0180, sweep_packet_size,
-		      7, 1, datasize, sweep_packet);
-	    break;
-	default:
-	  snprintf(syslog_message,message_size,"ppa_print_sweep(): %s",
-		   gMessages[E_PPA_UNKNOWN]);
-	  wrap_syslog (LOG_CRIT,"%s",syslog_message); 
-
-	    return;
-	}
-    }
+    break;
+  case HP1000:
+    scp3_put (printer->fptr, 0x0180, sweep_packet_size,
+	      7, 1, datasize, sweep_packet);
+    break;
+  default:
+    snprintf(syslog_message,message_size,"ppa_print_sweep(): %s",
+	     gMessages[E_PPA_UNKNOWN]);
+    wrap_syslog (LOG_CRIT,"%s",syslog_message); 
+    
+    return;
+  }
 }
 
 
+
 void
 ppa_print_sweeps (ppaPrinter_t * printer, ppaSweepData_t * data)
 {
